const mongoose = require('mongoose');
const config = require('./config').db;
const Category = require('./models/Category');
const Product = require('./models/Product');
const User = require('./models/User');

mongoose.connect(config.host + '/' + config.database);

const db = mongoose.connection;

db.once('open', async () => {
    try {
        await db.dropCollection('categories');
        await db.dropCollection('products');
        await db.dropCollection('users');
    } catch (e) {
        console.log('Collections were not present, skipping drop...');
    }

    const [cpuCategory, hddCategory, monitorCategory] = await Category.create([
        {
            title: "CPU's",
            description: "Central processor units",
        },
        {
            title: "HDDs",
            description: "Hard disk drives",
        },
        {
            title: "Monitors",
            description: "Monitors for office and home",
        }
    ]);

    await Product.create([
        {
            title: "Ryzen 7",
            category: cpuCategory._id,
            price: 300
        },
        {
            title: "HDD seagate 1TB",
            category: hddCategory._id,
            price: 200
        },
        {
            title: "Lenovo Legion 7",
            category: monitorCategory._id,
            price: 150
        },
    ]);

    await User.create([
        {
            username: "user",
            password: "qwerty",
            token: null,
            role: 'user'
        },
        {
            username: "admin",
            password: "qwerty",
            token: null,
            role: 'admin'
        }
    ]);

    db.close();
});
