const router = require('express').Router();
const User = require('../models/User');
const auth = require("../middleware/auth");
const config = require('../config');
const axios = require("axios");
const {OAuth2Client} = require("google-auth-library");
const {nanoid} = require('fix-esm').require("nanoid");

router.post('/', async (req, res) => {
    try {
        const user = new User({
            username: req.body.username,
            password: req.body.password
        });

        user.generateToken();
        await user.save();

        res.send(user);
    } catch (e) {
        res.status(400).send(e);
    }
});

router.put('/', auth, async (req, res) => {
    const user = req.user;

    if (
        req.body.oldPassword.trim()
        &&
        !await user.checkPassword(req.body.oldPassword.toString())
    ) {
        return res
            .status(400)
            .send({error: 'Old password incorrect'});
    }

    try {
        user.password = req.body.password || user.password;
        user.username = req.body.username || user.username;
        await user.save();

        res.send(user);
    } catch (e) {
        res
            .status(400)
            .send({error: e.message});
    }
});

router.post('/login', async (req, res) => {
    const user = await User.findOne({username: req.body.username});

    if (!user) return res
        .status(400)
        .send({error: 'Username or password incorrect'});

    if (!await user.checkPassword(req.body.password.toString())) return res
        .status(400)
        .send({error: 'Username or password incorrect'});

    try {
        user.generateToken();
        await user.save();

        res.send(user);
    } catch (e) {
        res
            .status(400)
            .send({error: e.message});
    }
});

router.get('/profile', auth, async (req, res) => {
    res.send(req.user);
});

router.delete('/logout', auth, async (req, res) => {
    req.user.token = null;
    req.user.save();

    res.send({message: 'Success'});
});

router.post('/facebookLogin', async (req, res) => {
   const inputToken = req.body.accessToken;
   const accessToken = config.facebook.appId + '|' + config.facebook.secretApp;
   const debugTokenUrl = `https://graph.facebook.com/debug_token?input_token=${inputToken}&access_token=${accessToken}`;

   try {
       const response = await axios.get(debugTokenUrl).then(res => res.data.data);

       if (response.error) {
           return res.status(401).send({message: 'Facebook token incorrect'});
       }

       if (req.body.id !== response.user_id) {
           return res.status(401).send({message: 'Wrong user ID'});
       }

       let user = await User.findOne({facebookId: req.body.id});

       if (!user) {
           user = new User({
               username: req.body.email,
               facebookId: req.body.id,
               displayName: req.body.name,
               password: nanoid(),
           });
       }

       user.generateToken();
       await user.save();

       return res.send(user);
   } catch (e) {
       return res.status(400).send({message: 'Facebook token incorrect'});
   }
});

router.post('/googleLogin', async (req, res) => {
    const idToken = req.body.tokenObj.id_token;
    const profileData = req.body.profileObj;

    try {
        const client = new OAuth2Client();

        const ticket = await client.verifyIdToken({
            idToken: idToken,
            audience: config.google.appId
        });
        const payload = ticket.getPayload();

        if (payload.error) {
            return res.status(401).send({message: 'Google token incorrect'});
        }

        if (profileData.googleId !== payload.sub) {
            return res.status(401).send({message: 'Wrong user ID'});
        }

        let user = await User.findOne({googleId: profileData.googleId});

        if (!user) {
            user = new User({
                username: profileData.email,
                googleId: profileData.googleId,
                displayName: profileData.name,
                password: nanoid(),
            });
        }

        user.generateToken();
        await user.save();

        return res.send(user);
    } catch (e) {
        return res.status(400).send({message: 'Google token incorrect'});
    }
});

module.exports = router;
