import {useState} from "react";
import {Button, Grid} from "@mui/material";
import FileInput from "../../UI/Form/FileInput/FileInput";
import FormElement from "../../UI/Form/FormElement/FormElement";

const ProductForm = ({createProductHandler, categories}) => {
    const [state, setState] = useState({
        title: "",
        price: "",
        description: "",
        category: "",
        image: ""
    });

    const submitFormHandler = e => {
        e.preventDefault();

        const formData = new FormData();

        Object.keys(state).forEach(key => {
            formData.append(key, state[key]);
        });

        createProductHandler(formData);
    };

    const inputChangeHandler = e => {
        const name = e.target.name;
        const value = e.target.value;

        setState(prevState => {
            return {...prevState, [name]: value};
        });
    };

    const onFileChangeHandler = e => {
        const file = e.currentTarget.files[0];
        const name = e.currentTarget.name;

        setState(prevState => {
            return {...prevState, [name]: file};
        });
    };

    return (
        <form
            autoComplete="off"
            onSubmit={submitFormHandler}
        >
            <Grid container direction="column" spacing={2}>
                <FormElement
                    id="title"
                    label="Title"
                    value={state.title}
                    onChange={inputChangeHandler}
                    name="title"
                />
                <FormElement
                    id="price"
                    label="Price"
                    value={state.price}
                    onChange={inputChangeHandler}
                    name="price"
                />
                <FormElement
                    multiline={true}
                    rows={3}
                    id="description"
                    label="Description"
                    value={state.description}
                    onChange={inputChangeHandler}
                    name="description"
                />
                <FormElement
                    id="category"
                    label="Category"
                    value={state.category}
                    onChange={inputChangeHandler}
                    name="category"
                    select={true}
                    options={categories}
                />
                <FileInput
                    onChange={onFileChangeHandler}
                    name="image"
                    label="Image"
                />
                <Grid item xs>
                    <Button type="submit" color="primary" variant="contained">Create</Button>
                </Grid>
            </Grid>
        </form>
    );
};

export default ProductForm;
